@@ -12,6 +12,7 @@
 #include  <stdlib.h>
 #endif
 
+#include "config.h"
 #include "fetchmail.h"
 
 #define HEADER_END(p)	((p)[0] == '\n' && ((p)[1] != ' ' && (p)[1] != '\t'))
@@ -49,6 +50,9 @@ const char *host;	/* server hostname */
     }
 
 #ifndef TESTMAIN
+    if (outlevel >= O_DEBUG)
+	error_build(0, 0, "About to rewrite %s", buf);
+#else
     /* make room to hack the address; buf must be malloced */
     for (cp = buf; *cp; cp++)
 	if (*cp == ',' || isspace(*cp))
@@ -170,6 +174,10 @@ const char *host;	/* server hostname */
 	}
     }
 
+#ifndef TESTMAIN
+    if (outlevel >= O_DEBUG)
+	error_complete(0, 0, "Rewritten version is %s", buf);
+#endif /* TESTMAIN */
     return(buf);
 }
 
